import { useState, useEffect } from "react";
/**
 * Hook to sync single search param with the URL.
 *
 * Initialise the hook by providing the param-name (key)
 * and it will try to get the corresponding value from
 * the URL if available.
 *
 * Use setter function to update the value and same should
 * be reflected in the URL. Setting empty value will remove
 * the param from the URL.
 *
 * @param paramName Key used for referring the value.
 * @param fallbackParamValue Value assigned to param if not already set.
 * @returns [value, setValue]
 *
 * @example
 * ```
 * const [userId, setUserId] = useSearchParamState("userId");
 * ```
 */
export default function useSearchParamState(paramName, fallbackParamValue) {
    var getDefaultParamValue = function () {
        return getURLSearchParams().get(paramName) || (fallbackParamValue !== null && fallbackParamValue !== void 0 ? fallbackParamValue : null);
    };
    var state = useState(function () {
        return getDefaultParamValue();
    });
    useEffect(function () {
        var urlSearchParams = getURLSearchParams();
        var newValue = state[0] || fallbackParamValue;
        if (newValue)
            urlSearchParams.set(paramName, newValue);
        else
            urlSearchParams.delete(paramName);
        replaceUrlWithNewSearchParams(urlSearchParams);
    }, [paramName, state[0], fallbackParamValue]);
    return state;
}
function getURLSearchParams() {
    var urlSearchString = typeof window !== "undefined" ? window.location.search : "";
    return new URLSearchParams(urlSearchString);
}
function replaceUrlWithNewSearchParams(urlSearchParams) {
    var _a, _b;
    var searchParamsString = urlSearchParams.toString();
    var newUrl = getCurrentUrl(false) + (searchParamsString ? "?" + searchParamsString : "");
    if (typeof window !== "undefined") {
        (_b = (_a = window.history).replaceState) === null || _b === void 0 ? void 0 : _b.call(_a, undefined, window.document.title, newUrl);
    }
}
function getCurrentUrl(withSearchParams) {
    if (withSearchParams === void 0) { withSearchParams = true; }
    if (withSearchParams)
        return window.location.href;
    if (typeof window !== "undefined") {
        var url = window.location.origin + window.location.pathname;
        if (window.location.hash)
            url += window.location.hash;
        return url;
    }
    return "";
}
